<?php

namespace Omnikassa\Payment\Components\OmnikassaPayment;
use Exception;

class OmnikassaService
{
	private $sRefreshToken = false;
	private $sSigningKey = false;
	private $bTestMode = false;
	private $sCachePath = false;
	private $sPaymentMethod = false;
	private $sLanguageCode = false;

	private $aCustomerShippingData = array();
	private $aCustomerBillingData = array();
	private $aAdditionalCustomerData = array();
	private $aProductData = array();

	private $sReturnUrl = false;
	private $sReportUrl = false;

	private $sOrderId = false;
	private $fAmount = false;
	private $sCurrencyCode = 'EUR';
	private $sDescription = false;
	private $sAccessToken = false;

	private $aTransaction = false;
	private $aTransactionResults = false;

	public function __construct($sRefreshToken = false, $sSigningKey = false)
	{
		$this->sRefreshToken = $sRefreshToken;
		$this->sSigningKey = $sSigningKey;
	}

	public function setRefreshToken($sRefreshToken)
	{
		$this->sRefreshToken = $sRefreshToken;
		return true;
	}

	public function setSigningKey($sSigningKey)
	{
		$this->sSigningKey = $sSigningKey;
		return true;
	}

	// Should point to directory where cache is strored
	public function setCachePath($sPath = false)
	{
		$this->sCachePath = $sPath;
	}

	public function setLanguageCode($sLanguageCode = false)
	{
		if(is_bool($sLanguageCode))
		{
			$this->sLanguageCode = false;
			return true;
		}
		elseif(is_string($sLanguageCode))
		{
			$sLanguageCode = strtolower(substr($sLanguageCode, 0, 2));

			if(in_array($sLanguageCode, array('nl', 'fr', 'de', 'es', 'en')))
			{
				$this->sLanguageCode = $sLanguageCode;
				return true;
			}
		}

		return false;
	}

	public function setPaymentMethod($sPaymentMethod = false)
	{
		if(is_bool($sPaymentMethod))
		{
			$this->sPaymentMethod = false;
			return true;
		}
		elseif(is_string($sPaymentMethod))
		{
			$sPaymentMethod = strtolower($sPaymentMethod);

			if(in_array($sPaymentMethod, array('afterpay', 'ideal', 'maestro', 'mastercard', 'paypal', 'visa', 'v_pay', 'bancontact')))
			{
				$this->sPaymentMethod = $sPaymentMethod;
				return true;
			}
		}

		return false;
	}

	public function setTestmode($bEnabled = true)
	{
		return ($this->bTestMode = $bEnabled);
	}


	public function setOrder($sOrderId, $sDescription = false)
	{
		$this->sOrderId = $sOrderId;
		$this->sDescription = $sDescription;

		if(empty($this->sDescription))
		{
			$this->sDescription = 'Webshop bestelling ' . $this->sOrderId;
		}

		return true;
	}


	public function setOrderAmount($fAmount)
	{
		$this->fAmount = $fAmount;

		return true;
	}

	public function setCurrencyCode($sCurrencyCode = false)
	{
		if(is_bool($sCurrencyCode))
		{
			$this->sCurrencyCode = false;
			return true;
		}
		elseif(is_string($sCurrencyCode))
		{
			$sCurrencyCode = strtoupper(substr($sCurrencyCode, 0, 3));

			if(in_array($sCurrencyCode, array('EUR')))
			{
				$this->sCurrencyCode = $sCurrencyCode;
				return true;
			}
		}

		return false;
	}


	public function setReturnUrl($sReturnUrl = false)
	{
		$this->sReturnUrl = $sReturnUrl;
		return true;
	}

/*
	public function setReportUrl($sReportUrl = false)
	{
		$this->sReportUrl = $sReportUrl;
		return true;
	}
*/

	public function setAccessToken()
	{
		$sUrl = 'https://betalen.rabobank.nl/omnikassa-api' . ($this->bTestMode ? '-sandbox' : '') . '/gatekeeper/refresh';

		$sResponse = $this->doHttpRequest($sUrl, '', array('Expect:', 'Authorization: Bearer ' . $this->sRefreshToken));

		if(!empty($sResponse))
		{
			$aToken = json_decode($sResponse, true);

			if(sizeof($aToken) && isset($aToken['token']))
			{
				$this->sAccessToken = $aToken['token'];
				return true;
			}
		}

		return false;
	}

	public function setCustomerShippingData($aCustomerShippingData)
	{
		$this->aCustomerShippingData = $aCustomerShippingData;
	}

	public function setCustomerBillingData($aPaymentCustomerData)
	{
		$this->aPaymentCustomerData = $aPaymentCustomerData;
	}


	public function setProductData($aProductData)
	{
		$this->aProductData = $aProductData;
	}

	public function setAdditionalCustomerData($aAdditionalCustomerData)
	{
		$this->aAdditionalCustomerData = $aAdditionalCustomerData;
	}

	public function getTransaction()
	{
		if(empty($this->sSigningKey))
		{
			$this->aTransaction = array('error' => array('message' => 'No Signing Key found.'));
			return false;
		}
		elseif(empty($this->sAccessToken))
		{
			$this->aTransaction = array('error' => array('message' => 'No Access token could be generated.'));
			return false;
		}
		elseif(empty($this->sOrderId))
		{
			$this->aTransaction = array('error' => array('message' => 'No order ID found.'));
			return false;
		}
		elseif(empty($this->fAmount))
		{
			$this->aTransaction = array('error' => array('message' => 'No amount found.'));
			return false;
		}
		elseif($this->fAmount < 0.29)
		{
			$this->aTransaction = array('error' => array('message' => 'Amount ' . number_format($this->fAmount, 2, ',', '') . ' is to small to process order #' . $this->sOrderId . '.'));
			return false;
		}
		elseif(empty($this->sReturnUrl))
		{
			$this->aTransaction = array('error' => array('message' => 'No return URL found.'));
			return false;
		}

		$aRequest['timestamp'] = date('c', time());
		$aRequest['merchantOrderId'] = $this->sOrderId;
		$aRequest['amount'] = array();
		$aRequest['amount']['currency'] = 'EUR';
		$aRequest['amount']['amount'] = $this->fAmount;

		if(!empty($this->sLanguageCode))
		{
			$aRequest['language'] = $this->sLanguageCode;
		}
		else
		{
			$aRequest['language'] = 'en';
		}

		$aRequest['description'] = $this->sDescription;
		$aRequest['merchantReturnURL'] = $this->sReturnUrl;

		// Optional
		$aRequest['paymentBrand'] = strtoupper($this->sPaymentMethod);
		$aRequest['paymentBrandForce'] = 'FORCE_ALWAYS';

		if(!empty($this->aProductData))
		{
			$aRequest['orderItems'] = $this->aProductData;
		}

		if(!empty($this->aCustomerShippingData))
		{
			$aRequest['shippingDetail'] = $this->aCustomerShippingData;
		}

		if(!empty($this->aPaymentCustomerData))
		{
			$aRequest['billingDetail'] = $this->aPaymentCustomerData;
		}

		if(!empty($this->aAdditionalCustomerData))
		{
			$aRequest['customerInformation'] = $this->aAdditionalCustomerData;
		}


		$sApiUrl = 'https://betalen.rabobank.nl/omnikassa-api' . ($this->bTestMode ? '-sandbox' : '') . '/order/server/api/v2/order';
		$sPostData = json_encode($aRequest);

		$sResponse = $this->doHttpRequest($sApiUrl, $sPostData, array('Expect:', 'Authorization: Bearer ' . $this->sAccessToken));

		if(!empty($sResponse))
		{
			$this->aTransaction = json_decode($sResponse, true);

			if($this->aTransaction)
			{
				if(isset($this->aTransaction['omnikassaOrderId']) && isset($this->aTransaction['redirectUrl']))
				{
					return true;
				}
				else
				{
					$this->aTransaction = array('error' => array('message' => $sResponse));
				}
			}
			else
			{
				$this->aTransaction = array('error' => array('message' => 'Cannot decode JSON response (See logs).'));
			}
		}
		else
		{
			$this->aTransaction = array('error' => array('message' => 'No response received from Rabo OmniKassa (See logs).'));
		}

		return false;
	}


	public function getTransactionUrl()
	{
		if(!empty($this->aTransaction['redirectUrl']))
		{
			return $this->aTransaction['redirectUrl'];
		}

		return false;
	}

	public function getTransactionId()
	{
		if(!empty($this->aTransaction['omnikassaOrderId']))
		{
			return $this->aTransaction['omnikassaOrderId'];
		}

		return false;
	}

	public function getError()
	{
		if(!empty($this->aTransaction['error']['message']))
		{
			return $this->aTransaction['error']['message'];
		}

		return false;
	}

	public function getNotificationData($sAuthToken)
	{
		if(!empty($sAuthToken))
		{
			$sApiUrl = 'https://betalen.rabobank.nl/omnikassa-api' . ($this->bTestMode ? '-sandbox' : '') . '/order/server/api/events/results/merchant.order.status.changed';

			$sResponse = $this->doHttpRequest($sApiUrl, '', array('Expect:', 'Authorization: Bearer ' . $sAuthToken));

			if(!empty($sResponse))
			{
				return $sResponse;
			}
		}

		return '';
	}


	public function checkResponse($sOmniKassaStatus)
	{
		if(!empty($sOmniKassaStatus))
		{
			if(strcasecmp($sOmniKassaStatus, 'IN_PROGRESS') === 0)
			{
				return 'PENDING';
			}
			elseif(strcasecmp($sOmniKassaStatus, 'COMPLETED') === 0)
			{
				return 'SUCCESS';
			}
			elseif(strcasecmp($sOmniKassaStatus, 'CANCELLED') === 0)
			{
				return 'CANCELLED';
			}
			elseif(strcasecmp($sOmniKassaStatus, 'EXPIRED') === 0)
			{
				return 'FAILURE';
			}

		}

		return '';
	}

	public function isValidRequest($sHashString = false, $sSignature)
	{
		if(!empty($sHashString))
		{
			$sHash = hash_hmac('sha512', $sHashString, base64_decode($this->sSigningKey));

			if(strcmp($sHash, $sSignature) === 0)
			{
				return true;
			}
		}

		return false;
	}

	public function doHttpRequest($sUrl, $sPostData, $aAdditionalHeaders = array())
    {
        $oCurl = curl_init();

		// Set default options
        curl_setopt($oCurl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($oCurl, CURLOPT_FOLLOWLOCATION, false);

        curl_setopt($oCurl, CURLOPT_URL, $sUrl);

		curl_setopt($oCurl, CURLOPT_TIMEOUT, 30);
		curl_setopt($oCurl, CURLOPT_HEADER, false);


		if(!empty($sPostData)) // JSON string
		{
			$aAdditionalHeaders[] = 'Content-Type: application/json';
		}

		if(is_array($aAdditionalHeaders) && sizeof($aAdditionalHeaders))
		{
			curl_setopt($oCurl, CURLOPT_HTTPHEADER, $aAdditionalHeaders);
		}

		if($sPostData != false)
		{
			curl_setopt($oCurl, CURLOPT_POST, true);
			curl_setopt($oCurl, CURLOPT_POSTFIELDS, $sPostData);
		}

        // GET/POST
        $sResponse = curl_exec($oCurl);

        // $sCurlInfo = curl_getinfo($oCurl);

        // check for curl errors
        if ($sResponse === false) {
            throw new Exception('Omnikassa API curl error: ' . curl_error($oCurl));
        }

        $aResponseData = json_decode($sResponse, true);

        // check for json_decode errors
        if ($aResponseData === null) {
            $aJsonErrors = [
                JSON_ERROR_NONE      => 'No error occurred',
                JSON_ERROR_DEPTH     => 'The maximum stack depth has been reached',
                JSON_ERROR_CTRL_CHAR => 'Control character issue, maybe wrong encoded',
                JSON_ERROR_SYNTAX    => 'Syntaxerror',
            ];

            throw new Exception('Omnikassa API json error: ' . (!empty($aJsonErrors[json_last_error()]) ? $aJsonErrors[json_last_error()] : 'JSON decode error') . ": " . $sResponse);
        }

        curl_close($oCurl);

        return $sResponse;
    }


}
