<?php

namespace Omnikassa\Payment\Controller\Api;

use Shopware\Core\Framework\Routing\Annotation\RouteScope;
use Shopware\Core\Framework\Validation\DataBag\RequestDataBag;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\Routing\Annotation\Route;

use Omnikassa\Payment\Components\OmnikassaPayment\OmnikassaService;

/**
 * @RouteScope(scopes={"administration"})
 */
class ApiTestController
{
    /**
     * @Route(path="/api/v{version}/_action/code1-rok2/verify")
     */
    public function check(RequestDataBag $dataBag): JsonResponse
    {
        $bSandbox = $dataBag->get('Code1Rok2.config.sandbox');

        if($bSandbox) {
            $refreshToken = $dataBag->get('Code1Rok2.config.testRefreshtoken');
            $signingKey = $dataBag->get('Code1Rok2.config.testSigningkey');
        }
        else{
            $refreshToken = $dataBag->get('Code1Rok2.config.prodRefreshtoken');
            $signingKey = $dataBag->get('Code1Rok2.config.prodSigningkey');
        }
        
        $oOmnikassa = new OmnikassaService();
        $oOmnikassa->setRefreshToken($refreshToken);
        $oOmnikassa->setSigningKey($signingKey);
        $oOmnikassa->setTestmode($bSandbox);

        $oOmnikassa->setOrder(time());
		$oOmnikassa->setPaymentMethod('ideal');
		// $oOmnikassa->setLanguageCode($sLocale);

		// Amount
		$oOmnikassa->setOrderAmount(rand(100, 10000));
		$oOmnikassa->setCurrencyCode('EUR');

		// Set return URL
        $sReturnUrl = 'https://www.ideal-checkout.nl/ping.php';
		$oOmnikassa->setReturnUrl($sReturnUrl);

        $oOmnikassa->setAccessToken();

        if($oOmnikassa->getTransaction())
		{
            $success = true;
		}
		else {
            $success = false;
		}
        
        return new JsonResponse(['success' => $success]);
    }
}