<?php declare(strict_types=1);

namespace Omnikassa\Payment\Storefront\Controller;

use Shopware\Core\Checkout\Order\Aggregate\OrderTransaction\OrderTransactionStateHandler;
use Shopware\Core\Checkout\Order\Aggregate\OrderTransaction\OrderTransactionStates;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Criteria;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Filter\EqualsFilter;

use Shopware\Storefront\Controller\StorefrontController;

use Omnikassa\Payment\Components\OmnikassaPayment\OmnikassaService;
use Omnikassa\Payment\Helpers\OmnikassaHelper;

use Shopware\Core\Framework\Context;

use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Annotation\Route;
use Symfony\Component\Routing\RouterInterface;

use Shopware\Core\Framework\Routing\Annotation\RouteScope;
use Shopware\Core\System\SalesChannel\SalesChannelContext;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepositoryInterface;

use Throwable;
use Exception;

class NotifyController extends StorefrontController
{
    /** @var Request $request */
    private $request;
    /** @var EntityRepositoryInterface $orderRepository */
    private $orderRepository;
    /** @var OrderTransactionStateHandler $orderTransactionStateHandler*/
    private $orderTransactionStateHandler;
    /** @var Context $context */
    private $context;
    /** @var OmnikassaHelper $omnikassaHelper */
    private $omnikassaHelper;
    /** @var OmnikassaService $omnikassa */
    protected $omnikassa;

    /**
     * NotifyController constructor.
     * @param EntityRepositoryInterface $orderRepository
     * @param OmnikassaHelper $omnikassaHelper
     * @param OrderTransactionStateHandler $orderTransactionStateHandler
     */
    public function __construct( EntityRepositoryInterface $orderRepository,
                                 OmnikassaHelper $omnikassaHelper,
                                 OrderTransactionStateHandler $orderTransactionStateHandler
                                 )
    {
        $this->orderRepository = $orderRepository;
        $this->omnikassaHelper = $omnikassaHelper;
        $this->request = new Request($_GET, $_POST, array(), $_COOKIE, $_FILES, $_SERVER);
        $this->context = \Shopware\Core\Framework\Context::createDefaultContext();
        $this->orderTransactionStateHandler = $orderTransactionStateHandler;
    }

    /**
     * @RouteScope(scopes={"storefront"})
     * @Route("/omnikassa/notify", name="omnikassa_notify", defaults={"csrf_protected": false}, methods={"POST"})
     * @return Response
     * @throws Exception
     */
    public function notify(Request $request, SalesChannelContext $salesChannelContext)
    {
		$sSalesChannelId = $salesChannelContext->getSalesChannel()->getId();

		$sJsonData = @file_get_contents('php://input');

		if(empty($sJsonData))
		{
            http_response_code(400);
            die('Invalid notification call, no data found');
		}
		else
		{
			$aPostData = json_decode($sJsonData, true);

			if(!empty($aPostData['authentication']) && !empty($aPostData['expiry']) && !empty($aPostData['eventName']) && !empty($aPostData['poiId']) && !empty($aPostData['signature']))
			{
				$sAuthToken = $aPostData['authentication'];
				$sExpiryString = $aPostData['expiry'];
				$sEventnameString = $aPostData['eventName'];
				$sPoiIdString = $aPostData['poiId'];
				$sSignatureString = $aPostData['signature'];

				// Controleer de signature met de data die is verstuurd
				$sHashString = $sAuthToken . ',' . $sExpiryString . ',' . $sEventnameString . ',' . $sPoiIdString;

				// Init Rabo OmniKassa
                $oOmnikassa = $this->omnikassaHelper->initOmnikassaClient($sSalesChannelId);

				if ($oOmnikassa->isValidRequest($sHashString, $sSignatureString)) {
                    $aResult = array();

					do {
                        $sResponse = $oOmnikassa->getNotificationData($sAuthToken);
                        $aResult = json_decode($sResponse, true);

                        if (isset($aResult['signature']) &&
                        isset($aResult['moreOrderResultsAvailable']) &&
                        isset($aResult['orderResults'])) {
                            $sSignature = $aResult['signature'];
                            $bMoreOrders = $aResult['moreOrderResultsAvailable'];

                            if ($bMoreOrders) {
                                $sMoreOrdersAvailable = 'true';
                            } else {
                                $sMoreOrdersAvailable = 'false';
                            }

                            $sHashString = $sMoreOrdersAvailable . ',';

                            // Validate total signature
                            foreach ($aResult['orderResults'] as $aTransaction) {
                                // Setup total hash string
                                $sTransactionString = $aTransaction['merchantOrderId'] . ',';
                                $sTransactionString .= $aTransaction['omnikassaOrderId'] . ',';
                                $sTransactionString .= $aTransaction['poiId'] . ',';
                                $sTransactionString .= $aTransaction['orderStatus'] . ',';
                                $sTransactionString .= $aTransaction['orderStatusDateTime'] . ',';
                                $sTransactionString .= $aTransaction['errorCode'] . ',';
                                $sTransactionString .= $aTransaction['paidAmount']['currency'] . ',';
                                $sTransactionString .= $aTransaction['paidAmount']['amount'] . ',';
                                $sTransactionString .= $aTransaction['totalAmount']['currency'] . ',';
                                $sTransactionString .= $aTransaction['totalAmount']['amount'];

                                $sHashString .= $sTransactionString . ',';
                            }

                            // Cut off last comma
                            $sHashString = substr($sHashString, 0, -1);

                            if ($oOmnikassa->isValidRequest($sHashString, $sSignature)) {
                                foreach ($aResult['orderResults'] as $aTransaction) {
                                    $sMerchantOrderId = $aTransaction['merchantOrderId'];
                                    $sOmniKassaStatus = $aTransaction['orderStatus'];


									$oOrderRepository = $this->orderRepository;
									$oCriteria = new Criteria();
									$oCriteria->addFilter(new EqualsFilter('orderNumber', $sMerchantOrderId))
										->addAssociation('transactions');
									$oOrder = $oOrderRepository->search($oCriteria, $this->context)->first();

									$oTransaction = $oOrder->getTransactions()->first();
									$sTransactionId = $oTransaction->getId();

									$sCurrentState = $oTransaction->getStateMachineState()->getTechnicalName();

									if(!in_array($sCurrentState , [
										OrderTransactionStates::STATE_OPEN,
										OrderTransactionStates::STATE_IN_PROGRESS,
										OrderTransactionStates::STATE_REMINDED])) {
										// Order already processed, continue
										continue;
									}

									if (strcmp($sOmniKassaStatus, 'COMPLETED') === 0) {
										$this->orderTransactionStateHandler->paid($sTransactionId, $this->context);

									} elseif (in_array($sOmniKassaStatus, array('CANCELLED', 'EXPIRED'))) {
										$this->orderTransactionStateHandler->cancel($sTransactionId, $this->context);
									}
								}
							}
						}
						else {
							http_response_code(400);
                            die('Invalid notification call, incorrect omnikassa data');
						}

					} while ($bMoreOrders);
				}
				else {
					http_response_code(400);
					die('Invalid notification call, signature mismatch');
				}
			}
			else
			{
				http_response_code(400);
				die('Invalid notification call, data incomplete');
			}
		}




		/*




		try {
            $orderRepo = $this->orderRepository;
            $criteria = new Criteria();
            $criteria->addFilter(new EqualsFilter('orderNumber', $orderNumber))
                ->addAssociation('transactions');
            $order = $orderRepo->search($criteria, $this->context)->first();

            $salesChannelId = $order->getSalesChannelId();
            $this->omnikassa = $this->omnikassaHelper->initOmnikassaClient($salesChannelId);

            $transaction = $order->getTransactions()->first();
            $transactionId = $transaction->getId();
            $customFields  = $order->getCustomFields();
            //$paymentMethodId = $transaction->getPaymentMethodId();
            //$paymentMethod = $this->omnikassaHelper->getPaymentMethodById($paymentMethodId);

            $orderTrxid = $customFields['omnikassa_trxid'];
            if(empty($orderTrxid)) {
                throw new Exception('No trxid');
            }

            if($orderTrxid != $trxid) {
                throw new Exception('Invalid order');
            }

            // Execute StatusRequest
            if(($ex = $this->omnikassa->StatusRequest($trxid)) < 0)
                throw new Exception('StatusRequest failed');

            // Omnikassa status set?
            if(empty($this->omnikassa->status)) {
                throw new Exception('No omnikassa status');
            }

            $currentState = $transaction->getStateMachineState()->getTechnicalName();

            if(!in_array($currentState , [OrderTransactionStates::STATE_OPEN, OrderTransactionStates::STATE_IN_PROGRESS, OrderTransactionStates::STATE_REMINDED ]) ) {
                throw new Exception('Order already processed!');
            }

            switch($this->omnikassa->status)
            {
                case "Cancelled":
                case "Expired":
                case "Denied":
                    if(OrderTransactionStates::STATE_OPEN != $currentState)
                        throw new Exception('Order already processed!');

                    $this->orderTransactionStateHandler->cancel($transactionId, $this->context);
                    break;
                case "Failure":
                    if(OrderTransactionStates::STATE_OPEN != $currentState)
                        throw new Exception('Order already processed!');

                    $this->transactionStateHandler->fail($transaction->getOrderTransaction()->getId(), $this->context);
                    break;
                case "Reversed":
                case "Refund":
                    $this->orderTransactionStateHandler->cancel($transactionId, $this->context);
                    break;
                case "Paid":
                case "Success":
                    if (OrderTransactionStates::STATE_PAID == $currentState)
                        throw new Exception('Order already processed!');

                    $this->orderTransactionStateHandler->paid($transactionId, $this->context);
                    break;
                case "Reservation":
                    if(OrderTransactionStates::STATE_OPEN != $currentState)
                        throw new Exception('Order already processed!');


                    if ($this->omnikassaHelper->getSetting($this->omnikassa->payment.'Makeinvoice', $salesChannelId) == true
                    ) {

                        $customFields = $transaction->getCustomFields() ?? [];
                        if (!isset($customFields['omnikassa_invoice'])) {

                            $this->omnikassa->InvoiceRequest($orderTrxid);

                            $customFields['omnikassa_invoice'] = true;
                            $transaction->setCustomFields($customFields);
                        }
                    }
                case "Pending":
                    if(OrderTransactionStates::STATE_OPEN != $currentState)
                        throw new Exception('Order already processed!');

                    $this->orderTransactionStateHandler->process($transactionId, $this->context);
                    break;
                case "Open":
                    if(OrderTransactionStates::STATE_OPEN != $currentState)
                        $this->orderTransactionStateHandler->reopen($transactionId, $this->context);
                    break;
                default:
                    throw new Exception('Status unknown');
            }

        } catch (InconsistentCriteriaIdsException $exception) {
            return $response->setContent('NG');
        } catch (Exception $exception) {
            return $response->setContent('NG');
        }

        return $response->setContent('OK');

		*/
    }
}